package Mail::DMARC::Report::View;
{
  $Mail::DMARC::Report::View::VERSION = '1.20130531';
}
use strict;
use warnings;

use Carp;

use parent 'Mail::DMARC::Base';

1;

# ABSTRACT: view DMARC report(s)

=pod

=head1 NAME

Mail::DMARC::Report::View - view DMARC report(s)

=head1 VERSION

version 1.20130531

=head1 DESCRIPTION

View DMARC reports

=head1 AUTHORS

=over 4

=item *

Matt Simerson <msimerson@cpan.org>

=item *

Davide Migliavacca <shari@cpan.org>

=back

=head1 CONTRIBUTOR

ColocateUSA.net <company@colocateusa.net>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2013 by ColocateUSA.com.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut

__END__
sub {}

